import io
import serial
import logging
from time import sleep

logger = logging.getLogger(__name__)

DEF_DELAY_BTW_CMD = 0.1  # sec - delay btw commands
DEF_DELAY_READ = 0.005    # sec - delay for reading of buffer

class KMulti:
    def __init__(self,port):
        self.port = port
        self.pd = serial.serial_for_url(port, timeout=1)
        self.pd.xonxoff = 'True'
        self.pd.baudrate = 9600
        # self.pd.bytesize = serial.EIGHTBITS
        # self.pd.parity = serial.PARITY_NONE
        self.sio = io.TextIOWrapper(io.BufferedRWPair(self.pd, self.pd),
                                    newline="\r")


    def write(self,cmd):
        logger.info('Send command: ' + cmd)
        self.sio.write(str(cmd) + '\r')
        self.sio.flush()
        sleep(DEF_DELAY_BTW_CMD)


    def read(self):
        buff=' '
        counter = 0
        while (buff[-1] != '\r'):
            counter += 1            
            buff += str(self.sio.readline())
            sleep(DEF_DELAY_READ)
            if (counter > 5):
                return False
        logger.info('Reading buffer: ' + buff)
        return buff


    def close_port(self):
        self.write(':SYST:LOC')
        self.pd.close()


    def init_device(self):        
        # check device connection
        self.write('*IDN?')
        idn = self.read()   # returns name/model/version of the connected device
        if (idn == False):
            return False
        self.write("*RST")
        # sleep(0.2)
        self.write('INIT')
        # self.write(':SYST:PRES')
        self.write(':SYST:BEEP:STAT OFF')        
        return True


    def set_res_mode(self):
        self.write(':CONF:RES')


    def set_volt_mode(self):
        self.write(':CONF:VOLT')


    def get_resistance(self):
        self.set_res_mode()
        self.write(':READ?')
        buff = self.read()
        if (buff == False):
            return 0
        value = float(buff.replace(' ','').replace('\x13',''))
        return value

    def get_voltage(self):
        self.set_volt_mode()
        self.write(':READ?')
        buff = self.read()
        if (buff == False):
            return 0
        value = float(buff.replace(' ','').replace('\x13',''))
        return value
        

     # R vs T ratio for TAO thermosensor
    def convert_to_temp(self,R):
        try:
            import numpy as np
        except:
            logger.error('numpy module is not installed!')
            return False
        # R=R-.16-.14-1.14+.24+1;
        return 25.+0.5*np.sqrt(-89549.4645432508+(5.039684199579497*(-109.10492760163322+R))/(1.885411358756482*10**-9.-1.6916393033018334*10**-12*R+7.110786321133254*10**-13*np.sqrt(268.5157052244801-R)*np.sqrt(31019.12828161577+(-64.45860641904099+R)*R))**(1./3.)+6.325937610085937*10**8.*(1.885411358756482*10**-9-1.6916393033018334*10**-12.*R+7.110786321133254*10**-13.*np.sqrt(268.5157052244801-R)*np.sqrt(31019.12828161577+(-64.45860641904099+R)*R))**(1./3.))-0.5*np.sqrt(-179098.9290865016-(5.039684199579497*(-109.10492760163322+R))/(1.885411358756482*10**-9-1.6916393033018334*10**-12*R+7.110786321133254*10**-13*np.sqrt(268.5157052244801-R)*np.sqrt(31019.12828161577+(-64.45860641904099+R)*R))**(1./3.)-6.325937610085937*10**8*(1.885411358756482*10**-9-1.6916393033018334*10**-12*R+7.110786321133254*10**-13*np.sqrt(268.5157052244801-1*R)*np.sqrt(31019.12828161577+(-64.45860641904099+R)*R))**(1./3.)+1.8551694513486145*10**9/np.sqrt(-89549.4645432508+(5.039684199579497*(-109.10492760163322+1*R))/(1.885411358756482*10**-9-1.6916393033018334*10**-12*R+7.110786321133254*10**-13*np.sqrt(268.5157052244801-1*R)*np.sqrt(31019.12828161577+(-64.45860641904099+R)*R))**(1./3.)+6.325937610085937*10**8*(1.885411358756482*10**-9-1.6916393033018334*10**-12*R+7.110786321133254*10**-13*np.sqrt(268.5157052244801-R)*np.sqrt(31019.12828161577+(-64.45860641904099+R)*R))**(1./3.)));